import { Iterable, Map, Record as ImmutableRecord } from 'immutable';

export interface Record<T extends Record<T>> extends Map<string, any> {
  set: (prop: string, val: any) => T;
  delete: (key: string) => T;
  remove: (key: string) => T;
  clear: () => T;
  update: {
    (updater: (value: T) => any): T;
    (key: string, updater: (value: any) => any): T;
    (key: string, notSetValue: any, updater: (value: any) => any): T;
  };
  merge: (obj: any) => T;
  mergeWith: (
    merger: (previous?: any, next?: any, key?: string) => any,
    obj: any,
  ) => T;
  mergeDeep: (obj: any) => T;
  mergeDeepWith: (
    merger: (previous?: any, next?: any, key?: string) => any,
    obj: any,
  ) => T;
  setIn: (keyPath: any[] | Iterable<any, any>, value: any) => T;
  deleteIn: (keyPath: Array<any> | Iterable<any, any>) => T;
  removeIn: (keyPath: Array<any> | Iterable<any, any>) => T;
  updateIn: {
    (keyPath: any[] | Iterable<any, any>, updater: (value: any) => any): T;
    (
      keyPath: any[] | Iterable<any, any>,
      notSetValue: any,
      updater: (value: any) => any,
    ): T
  };
  mergeIn: (keyPath: any[] | Iterable<any, any>, obj: any) => T;
  mergeDeepIn: (keyPath: any[] | Iterable<any, any>, obj: any) => T;
  withMutations: (mutator: (mutable: T) => any) => T;
  asMutable: () => T;
  asImmutable: () => T;
}

export function makeRecordType<O, R extends Record<R> & O>
  (defaultVal: O, val: O = null, name?: string): R {

  const RecordFactory = makeRecordFactory<O, R>(defaultVal, name);
  return val ? RecordFactory(val) : RecordFactory();
}

export function makeRecordFactory<O, R extends Record<R> & O>
  (obj: O, name?: string): (val?: O) => R {

  const iRecord = ImmutableRecord(obj, name);
  return (val: O = null) => {
    return new iRecord(val) as R;
  };
}
