import Action from 'commons/redux/Action';
import { Record, makeRecordFactory } from 'commons/util/immutable/Record';
import { Member } from 'member/api/model/Member';

export const ModuleName = 'signup';

export type REQUEST = '@overflow/member/signup/REQUEST';
export type REQUEST_SUCCESS = '@overflow/member/signup/REQUEST_SUCCESS';
export type REQUEST_FAILURE = '@overflow/member/signup/REQUEST_FAILURE';

export interface MemberPayload extends Member {
}

export interface State {
  isRegistered: boolean;
  member?: Member;
  error?: Error;
}

export interface StateRecord extends Record<StateRecord>, State {
}

const makeStateRecord = makeRecordFactory<State, StateRecord>({
  isRegistered: undefined,
  member: undefined,
  error: undefined,
});

export interface IActionTypes {
  REQUEST: REQUEST;
  REQUEST_SUCCESS: REQUEST_SUCCESS;
  REQUEST_FAILURE: REQUEST_FAILURE;
}
export const ActionTypes: IActionTypes = {
  REQUEST: '@overflow/member/signup/REQUEST',
  REQUEST_SUCCESS: '@overflow/member/signup/REQUEST_SUCCESS',
  REQUEST_FAILURE: '@overflow/member/signup/REQUEST_FAILURE',
};

export interface IActions {
  request: (member: Member) => Action<MemberPayload>;
  requestSuccess: (member: Member) => Action<Member>;
  requestFailure: (error: Error) => Action;
}
export const Actions: IActions = {
    request(member: Member): Action<MemberPayload> {
      return {
          type: ActionTypes.REQUEST,
          payload: member,
        };
    },

    requestSuccess(member: Member): Action<Member> {
      return {
          type: ActionTypes.REQUEST,
          payload: member,
        };
    },

    requestFailure(error: Error): Action {
      return {
          type: ActionTypes.REQUEST,
          error: error,
        };
    },
};

export function Reducers(state: StateRecord = makeStateRecord(), action: Action<MemberPayload | Error>): StateRecord {
  switch (action.type) {
    case ActionTypes.REQUEST_SUCCESS:
    {
      return state.merge({isRegistered: true, member: action.payload});
    }
    case ActionTypes.REQUEST_FAILURE:
    {
      return state.merge({isRegistered: false, error: action.error});
    }
    default:
      return state;
  }
}
